/*
 * Copyright (C) 2002  Carl M. Gregory - MC_8
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/* Method, 0= privmsg
 *         1= notice
 */
static void add_greet(char chan[CHANLEN+1], char mesg[MESGLEN+1], int meth) {
	greet_t *new, *greet;
	ContextNote(REPORT_BUG);
	greet = findgreet(chan);
	if (greet)
		return;
	new = nmalloc(sizeof(struct greet_table));
	if (!new)
		return;
	new->next = greetdata;
	greetdata = new;
	new->channel = nmalloc(strlen(chan) + 1);
	strcpy(new->channel, chan);
	new->message = nmalloc(strlen(mesg) + 1);
	strcpy(new->message, mesg);
	new->method = meth;
}

static void del_greet(greet_t *goner) {
	greet_t *tmp = NULL, *old = NULL;
	ContextNote(REPORT_BUG);
	for (tmp = greetdata; tmp; old = tmp, tmp = tmp->next) {
		if (tmp == goner) {
			if (old)
				old->next = tmp->next;
			else
				greetdata = tmp->next;
			nfree(tmp->channel);
			nfree(tmp->message);
			nfree(tmp);
			break;
		}
	}
}

static greet_t *findgreet(char *channel) {
	greet_t *greet;
	ContextNote(REPORT_BUG);
	for (greet = greetdata; greet; greet = greet->next)
		if (!strcasecmp(channel, greet->channel))
			return greet;
	return NULL;
}

static void chk_validchan() {
	greet_t *g;
  struct chanset_t *chan;
	ContextNote(REPORT_BUG);
	for (g = greetdata; g; g = g->next)
		if (!(chan = findchan_by_dname(g->channel)))
			del_greet(g);
}

static void load_datafile() {
	FILE *f = NULL;
	char buf[512+1], *s, chan[CHANLEN+1], msg[MESGLEN+1], type[2];
	int meth = 0;
	ContextNote(REPORT_BUG);
  f = fopen(datafile, "r");
  if (!f) {
		putlog(LOG_MISC, "*", "Cannot load mc_greet datafile, creating new one.");
		return;
	}
	msg[0] = 0;
	while (fgets(buf, sizeof(buf), f) != NULL) {
		if (buf[strlen(buf)-1] == '\n')
      buf[strlen(buf)-1] = 0;
    s = buf;
    strncpyz(type, newsplit(&s), sizeof(type));
    switch (type[0]) {
      case '!':
				strncpyz(chan, newsplit(&s), sizeof(chan));
      	meth = atoi(newsplit(&s));
        break;
      case '^':
        if (!msg[0])
          strncpyz(msg, s, sizeof(msg));
        else {
					strncat(msg, "\n", sizeof(msg));
          strncat(msg, s, sizeof(msg));
				}
        break;
      case '-':
      	add_greet(chan, msg, meth);
        msg[0] = 0;
        break;
    }
	}
	fclose(f);
}

static void save_datafile(int quiet) {
	greet_t *g;
	FILE *f = NULL;
	char s[125], mesg[MESGLEN+1], *mesg_;
	ContextNote(REPORT_BUG);
	if (!quiet)
		putlog(LOG_MISC, "*", "Saving greet file...");
	if (!datafile[0])
		return;
	sprintf(s, "%s~new", datafile);
	f = fopen(s, "w");
	if (f == NULL) {
		putlog(LOG_MISC, "*", "Error writing mc_greet datafile.");
		return;
	}
	chmod(s, 0600);
	fprintf(f, "# mc_greet datafile v1.0\n");
	for (g = greetdata; g; g = g->next) {
		fprintf(f, "! %s %d\n", g->channel, g->method);
		strncpyz(mesg, g->message, sizeof(mesg));
		for (mesg_ = strtok(mesg, "\n"); mesg_; mesg_ = strtok(NULL, "\n"))
			fprintf(f, "^ %s\n", mesg_);
		fprintf(f, "-\n");
	}
	fclose(f);
	unlink(datafile);
	movefile(s, datafile);
}
