/*
 * Copyright (C) 2002  Carl M. Gregory - MC_8
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define MAKING_MC_GREET
#define MODULE_NAME "mc_greet"
#define MODULE_VERSION "1.0"
#define MESGLEN 1024
#define CHANLEN 80
#define REPORT_BUG "Greet (module):  If this is the last context, report this bug to http://mc.purehype.net:81/bugzilla/"

#include "../module.h"
#include "../irc.mod/irc.h"

#undef global
static Function *global = NULL, *irc_funcs = NULL;

#include "mc_greet.h"
#include "regexp.c"
#include "mc_preset.c"

/* SVS section */
#define SVS_NAME		"greet_mod"
#define SVS_VERSION	"001000000000"
#define SVS_MODE		svs_mode
#define SVS_NOTIFY	svs_notify
#include "mc_svs.c"

#include "mc_mem.c"
#include "mc_delay.c"
#include "mc_data.c"
#include "mc_tclcmds.c"
#include "mc_on_dcc.c"
#include "mc_on_join.c"
#include "mc_misc.c"



static void mc_greet_hook_secondly() {
	check_delay();
}

static void mc_greet_hook_read_userfile() {
	load_datafile();
}

static void mc_greet_hook_userfile() {
	save_datafile(0);
	chk_validchan();
}

static void mc_greet_report(int idx, int details) {
	greet_t *tmp;
	char *msg, *msg_;
	ContextNote(REPORT_BUG);
	if (details) {
		dprintf(idx, "    Version: %s\n", MODULE_VERSION);
		dprintf(idx, "    Memory: %2d record%s, %4d byte%s (Greet structure)\n",
		        greet_recnum(), greet_recnum() == 1 ? " " : "s", greet_expmem(),
		        greet_expmem() == 1 ? " " : "s");
		dprintf(idx, "            %2d record%s, %4d byte%s (Delay structure)\n",
		        delay_recnum(), delay_recnum() == 1 ? " " : "s", delay_expmem(),
		        delay_expmem() == 1 ? " " : "s");
		dprintf(idx, "                        %4d byte%s (All of module)\n",
		        mc_greet_expmem(), mc_greet_expmem() == 1 ? " " : "s");
		dprintf(idx, "    -- Sending greet to %s.\n",
		        validuser ? "all users" : "only users not in bots userlist");
		dprintf(idx, "    -- Waiting %d seconds before sending a greet.\n", wait);
		dprintf(idx, "    -- Saving data to: %s\n", datafile);
		for (tmp = greetdata; tmp; tmp = tmp->next) {
			dprintf(idx, "    %-10s (%-7s):\n", tmp->channel,
			        tmp->method ? "NOTICE" : "PRIVMSG");
			msg_ = nmalloc(strlen(tmp->message));
			strcpy(msg_, tmp->message);
			for (msg = strtok(msg_, "\n"); msg; msg = strtok(NULL, "\n"))
				dprintf(idx, "      %s\n", msg);
			if (tmp->next)
				dprintf(idx, "\n");
		}
	}
}

static char *mc_greet_close() {
	ContextNote(REPORT_BUG);
	save_datafile(1);
	close_svs();
	del_hook(HOOK_SECONDLY, (Function) mc_greet_hook_secondly);
  del_hook(HOOK_READ_USERFILE, (Function) mc_greet_hook_read_userfile);
  del_hook(HOOK_USERFILE, (Function) mc_greet_hook_userfile);
	free_mc_greet();
	rem_tcl_ints(my_tcl_ints);
	rem_tcl_strings(my_tcl_strings);
	rem_tcl_commands(my_tcl_cmds);
	rem_builtins(H_dcc, my_dcc_cmds);
	rem_builtins(H_join, my_joins);
	module_undepend(MODULE_NAME);
	return NULL;
}

static Function mc_greet_table[] = {
	(Function) mc_greet_start,
	(Function) mc_greet_close,
	(Function) mc_greet_expmem,
	(Function) mc_greet_report
};

char *mc_greet_start(Function *global_funcs) {
	global = global_funcs;
	ContextNote(REPORT_BUG);
	module_register(MODULE_NAME, mc_greet_table, 1, 0);
	if (!(irc_funcs = module_depend(MODULE_NAME, "irc", 1, 0))) {
		module_undepend(MODULE_NAME);
		return "You need the irc module to use the mc_greet module.";
	}
	add_tcl_ints(my_tcl_ints);
	add_tcl_strings(my_tcl_strings);
	add_tcl_commands(my_tcl_cmds);
	add_builtins(H_dcc, my_dcc_cmds);
	add_builtins(H_join, my_joins);
	load_datafile();
	start_svs();
	add_hook(HOOK_SECONDLY, (Function) mc_greet_hook_secondly);
	add_hook(HOOK_READ_USERFILE, (Function) mc_greet_hook_read_userfile);
  add_hook(HOOK_USERFILE, (Function) mc_greet_hook_userfile);
	return NULL;
}

