/*
 * Copyright (C) 2002  Carl M. Gregory - MC_8
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


static int cmd_pls_greet(struct userrec *u, int idx, char *par) {
	char chan[CHANLEN+1], meth[7+1], mesg[MESGLEN+1], code[2];
	struct chanset_t *c;
	ContextNote(REPORT_BUG);
	strncpyz(chan, newsplit(&par), sizeof(chan));
	strncpyz(meth, newsplit(&par), sizeof(meth));
	strncpyz(mesg, par, sizeof(mesg));
	if ((!strcmp(mesg, "")          && strncpy(code, "1", sizeof(code))) ||
	    (!strchr(CHANMETA, chan[0]) && strncpy(code, "2", sizeof(code))) ||
	    (strcasecmp(meth, "PRIVMSG") &&
	     strcasecmp(meth, "NOTICE") && strncpy(code, "3", sizeof(code))) ||
	    (!(c = findchan(chan))      && strncpy(code, "4", sizeof(code))) ||
	    (!matchattr(u, addremove_access, c->dname) &&
	                                   strncpy(code, "5", sizeof(code))) ||
	    (findgreet(c->dname)        && strncpy(code, "6", sizeof(code)))) {
		switch (code[0]) {
			case '1':
				dprintf(idx, "Usage: +greet [%s]<channel> <method> <message>\n",
				        CHANMETA);
        break;
			case '2':
				dprintf(idx, "Invalid channel prefix, should be one of: %s\n",
				        CHANMETA);
				break;
			case '3':
				dprintf(idx, "Unknown +greet method: should be one of: %s\n",
				        "PRIVMSG NOTICE");
				break;
			case '4':
				dprintf(idx, "That channel doesn't exist!\n");
				break;
			case '5':
				dprintf(idx, "Insufficent access, denied.\n");
				break;
			case '6':
				dprintf(idx, "Greet already exists for %s.\n", chan);
				break;
    }
		return 0;
	}
	putlog(LOG_CMDS, "*", "#%s# +greet %s %s %s", dcc[idx].nick, chan,
	       meth, mesg);
	add_greet(c->dname, regsub("\\\\n", mesg, "\n", 0),
	          !strcasecmp(meth, "NOTICE") ? 1 : 0);
	dprintf(idx, "Added greet for %s.\n", chan);
	return 0;
}

static int cmd_mns_greet(struct userrec *u, int idx, char *par) {
	greet_t *greet;
	char chan[CHANLEN+1], code[2];
	struct chanset_t *c;
	ContextNote(REPORT_BUG);
	strncpyz(chan, newsplit(&par), sizeof(chan));
	if ((!strcmp(chan, "")          && strncpy(code, "1", sizeof(code))) ||
	    (!strchr(CHANMETA, chan[0]) && strncpy(code, "2", sizeof(code))) ||
	    (!(c = findchan(chan))      && strncpy(code, "3", sizeof(code))) ||
	    (!(greet = findgreet(chan)) && strncpy(code, "4", sizeof(code))) ||
	    (!matchattr(u, addremove_access, c->dname) &&
	                                   strncpy(code, "5", sizeof(code)))) {
		switch (code[0]) {
			case '1':
				dprintf(idx, "Usage: -greet [%s]<channel>\n", CHANMETA);
				break;
			case '2':
				dprintf(idx, "Invalid channel prefix, should be one of: %s\n",
				        CHANMETA);
				break;
			case '3':
				dprintf(idx, "That channel doesn't exist!\n");
				break;
			case '4':
				dprintf(idx, "No greet setup for %s.\n", chan);
				break;
			case '5':
				dprintf(idx, "Insufficent access, denied.\n");
				break;
		}
		return 0;
	}
	putlog(LOG_CMDS, "*", "#%s# delgreet %s", dcc[idx].nick, chan);
	del_greet(greet);
	dprintf(idx, "Removed greet for %s.\n", chan);
	return 0;
}

static int cmd_greets(struct userrec *u, int idx, char *par) {
	greet_t *greet;
	char chan[CHANLEN+1], mesg[MESGLEN+1], *mesg_, code[2];
	struct chanset_t *c;
	ContextNote(REPORT_BUG);
	strncpyz(chan, newsplit(&par), sizeof(chan));
	if (!strcmp(chan, "")) {
		if (!matchattr(u, view_access, NULL)) {
			dprintf(idx, "Insufficent access, denied.\n");
			return 0;
		}
		greet = greetdata;
	}	else {
		if ((!strchr(CHANMETA, chan[0]) && strncpy(code, "1", sizeof(code))) ||
	      (!(c = findchan(chan))      && strncpy(code, "2", sizeof(code))) ||
	      (!(greet = findgreet(chan)) && strncpy(code, "3", sizeof(code))) ||
	      (!matchattr(u, view_access, c->dname) &&
	                                     strncpy(code, "4", sizeof(code)))) {
			switch (code[0]) {
				case '1':
					dprintf(idx, "Invalid channel prefix, should be one of: %s\n",
					        CHANMETA);
					break;
				case '2':
					dprintf(idx, "That channel doesn't exist!\n");
					break;
				case '3':
					dprintf(idx, "No greet setup for %s.\n", chan);
					break;
				case '4':
					dprintf(idx, "Insufficent access, denied.\n");
					break;
			}
			return 0;
		}
	}
	putlog(LOG_CMDS, "*", "#%s# greets %s", dcc[idx].nick, chan);
	for (;greet; greet = greet->next) {
		c = findchan_by_dname(greet->channel);
		dprintf(idx, " %-10s (%-7s):\n", c->name,
		        greet->method ? "NOTICE" : "PRIVMSG");
		strncpyz(mesg, greet->message, sizeof(mesg));
		for (mesg_ = strtok(mesg, "\n"); mesg_; mesg_ = strtok(NULL, "\n"))
			dprintf(idx, "   %s\n", mesg_);
		if (greet->next)
			dprintf(idx, "\n");
	}
	return 0;
}

static cmd_t my_dcc_cmds[] = {
	{"+greet", "-|-", cmd_pls_greet, "mc_greet:+greet"},
	{"-greet", "-|-", cmd_mns_greet, "mc_greet:-greet"},
	{"greets", "-|-", cmd_greets, "mc_greet:greets"},
	{0, 0, 0, 0}
};