/*
 * Copyright (C) 2002  Carl M. Gregory - MC_8
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/* Personal note, inter-twine this with key parts of the module:
 *
 * Presets:
 *		static int svs_mode = 2;
 *		static char svs_notify[512] = "";
 *		static char svs_notefile[120] = "";
 *		static tcl_ints my_tcl_ints[] = {
 *			{"mc_greet-svs-mode", &svs_mode, 0},
 *			{0, 0, 0}
 *		};
 *		static tcl_strings my_tcl_strings[] = {
 *			{"mc_greet-svs-notify",	svs_notify,		512,	0},
 *			{"notefile",						svs_notefile,	120,	0},
 *			{0, 0, 0, 0}
 *		};
 *
 * Configs file:
 *		## MC SVS (Script Version Service) (Module) v1.0 ##
 *		# Do you want the bot to connection to MC_8's SVS server to see if there's
 *		# a newer version of this module avilable?  It will only tell you, it's up
 *		# to you to get the newest version and compile it.
 *		#   0 = No, do not enable SVS.
 *		#   1 = Yes, but only inform me of non beta versions.
 *		#   2 = Yes, inform me of all new versions (beta and stable).
 *		set mc_greet-svs-mode 2
 *
 *		# Upon finding a newer version, whom do you want to notify?  The
 *		# notification is # sent via 'note'.  Seperate each user with a space.
 *		# For those whom know TCL; do not put this in list format, keep it in
 *    # string format.
 *		set mc_greet-svs-notify "MC_8"
 *
 * Include:
 *		// SVS section
 *		#define SVS_NAME		"z_svs_mod_test"
 *		#define SVS_VERSION	"001000000000"
 *		#define SVS_MODE		svs_mode
 *		#define SVS_NOTIFY	svs_notify
 *		#include "mc_svs.c"
 *
 * <module>_close:
 *		close_svs();
 *
 * <module>_start:
 *		start_svs();
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include "mc_svs.h"

#ifndef _REGEXP_H_
#include "regexp.c"
#endif


#define SVS_HOST		"mc.purehype.net"
#define SVS_PORT		81
#define SVS_GET			"GET /?a=svs_c&n=%s&v=%s HTTP/1.0\nHost: %s:%i\n\n"
#define SVS_DATALEN	1024


static int isready_w(int fd) {
	int rc;
	fd_set fds;
	struct timeval tv;

	FD_ZERO(&fds);
	FD_SET(fd,&fds);
	tv.tv_sec = tv.tv_usec = 0;

	rc = select(fd+1, NULL, &fds, NULL, &tv);
	if (rc < 0)
		return 0;

	return FD_ISSET(fd,&fds) ? 1 : 0;
}

static int isready_r(int fd) {
	int rc;
	fd_set fds;
	struct timeval tv;

	FD_ZERO(&fds);
	FD_SET(fd,&fds);
	tv.tv_sec = tv.tv_usec = 0;

	rc = select(fd+1, &fds, NULL, NULL, &tv);
	if (rc < 0)
		return 0;

	return FD_ISSET(fd,&fds) ? 1 : 0;
}

static void do_svs() {
	struct hostent *host_t;
	struct sockaddr_in localAddr_t, servAddr_t;
	int temp_int_1, temp_int_2;
	char temp_char_SVS_DATALEN_1[SVS_DATALEN],
	     *temp_char_pointer_1,
	     *temp_char_pointer_2,
	     *temp_char_pointer_3,
	     *temp_char_pointer_4,
 	     *temp_char_pointer_5,
	     *temp_char_array_1[8];
	ContextNote(REPORT_BUG);

	if ((!SVS_MODE) || (!svs_notefile[0]))
		return;

	host_t = gethostbyname(SVS_HOST);
	if(host_t == NULL)
		return;

	servAddr_t.sin_family = host_t->h_addrtype;
	memcpy((char *) &servAddr_t.sin_addr.s_addr,
		host_t->h_addr_list[0], host_t->h_length);
	servAddr_t.sin_port = htons(SVS_PORT);

	socket_id = socket(AF_INET, SOCK_STREAM, 0);
	if(socket_id < 0)
		return;

	localAddr_t.sin_family = AF_INET;
	localAddr_t.sin_addr.s_addr = htonl(INADDR_ANY);
	localAddr_t.sin_port = htons(0);

	temp_int_1 = bind(socket_id, (struct sockaddr *) &localAddr_t,
		sizeof(localAddr_t));
  if(temp_int_1 < 0)
  	return;

	temp_int_1 = connect(socket_id, (struct sockaddr *) &servAddr_t,
		sizeof(servAddr_t));
  if(temp_int_1 < 0)
		return;

	add_hook(HOOK_SECONDLY, (Function) do_svs_write);
	return;
}

static void do_svs_write() {
	struct hostent *host_t;
	struct sockaddr_in localAddr_t, servAddr_t;
	int temp_int_1, temp_int_2;
	char temp_char_SVS_DATALEN_1[SVS_DATALEN],
	     *temp_char_pointer_1,
	     *temp_char_pointer_2,
	     *temp_char_pointer_3,
	     *temp_char_pointer_4,
 	     *temp_char_pointer_5,
	     *temp_char_array_1[8];
	ContextNote(REPORT_BUG);

	sprintf(temp_char_SVS_DATALEN_1, SVS_GET, SVS_NAME, SVS_VERSION, SVS_HOST,
		SVS_PORT);

	if (!isready_w(socket_id))
		return;
	del_hook(HOOK_SECONDLY, (Function) do_svs_write);

	temp_int_1 = send(socket_id, temp_char_SVS_DATALEN_1,
		strlen(temp_char_SVS_DATALEN_1) + 1, 0);
	if(temp_int_1 < 0) {
		close(socket_id);
		return;
	}

	add_hook(HOOK_SECONDLY, (Function) do_svs_read);
	return;
}

static void do_svs_read() {
	struct hostent *host_t;
	struct sockaddr_in localAddr_t, servAddr_t;
	int temp_int_1, temp_int_2;
	char temp_char_SVS_DATALEN_1[SVS_DATALEN],
	     temp_char_SVS_DATALEN_2[SVS_DATALEN],
	     *temp_char_pointer_1,
	     *temp_char_pointer_2,
	     *temp_char_pointer_3,
	     *temp_char_pointer_4,
 	     *temp_char_pointer_5,
	     *temp_char_array_1[8];
	ContextNote(REPORT_BUG);

	if (!isready_r(socket_id))
		return;

	del_hook(HOOK_SECONDLY, (Function) do_svs_read);

	temp_int_1 = recv(socket_id, temp_char_SVS_DATALEN_1, SVS_DATALEN, 0);
	close(socket_id);
	if (temp_int_1 < 0)
		return;
	else if (temp_int_1 == 0)
		return;

	/* temp_char_SVS_DATALEN_1  == Incomming message. */

  for (temp_char_pointer_3 = strtok_r(temp_char_SVS_DATALEN_1, "\n",
  	&temp_char_pointer_1);
       temp_char_pointer_3;
       temp_char_pointer_3 = strtok_r(NULL, "\n", &temp_char_pointer_1)) {
		temp_char_pointer_3[strlen(temp_char_pointer_3)-1] = '\0';
		temp_char_pointer_4 = strtok_r(temp_char_pointer_3, " ",
			&temp_char_pointer_2);
		temp_char_pointer_5 = strtok_r(NULL, "\0", &temp_char_pointer_2);
		if (!temp_char_pointer_4)
			continue;
		if (temp_char_pointer_4[0] == '1') {
			temp_int_1 = atoi(&temp_char_pointer_4[2]);
			temp_char_array_1[temp_int_1] = temp_char_pointer_5;
		}
	}

	/* temp_char_array_1  == RFC Array */

	strcpy(temp_char_SVS_DATALEN_1, SVS_VERSION);
	for (temp_int_1 = 0, temp_int_2 = 0;
	     temp_char_SVS_DATALEN_1[temp_int_2];
	     temp_int_2++) {
		if (((SVS_MODE == 1) && (temp_int_2 == 9)) || (temp_int_2 == 12) ||
			(temp_char_array_1[1][temp_int_2] <
				temp_char_SVS_DATALEN_1[temp_int_2]))
			break;
		if (temp_char_array_1[1][temp_int_2] >
			temp_char_SVS_DATALEN_1[temp_int_2]) {
			temp_int_1++;
			break;
		}
	}

	/* temp_char_array_1 = the array of rfc's and mesg's  */
	if ((temp_int_1) && (temp_char_array_1[7])) {
		strcpy(temp_char_SVS_DATALEN_2, regsub("%0", temp_char_array_1[7],
			temp_char_array_1[0] ? temp_char_array_1[0] : "(null)", 0));
		strcpy(temp_char_SVS_DATALEN_2, regsub("%1", temp_char_SVS_DATALEN_2,
			temp_char_array_1[1] ? temp_char_array_1[1] : "(null)", 0));
		strcpy(temp_char_SVS_DATALEN_2, regsub("%2", temp_char_SVS_DATALEN_2,
			temp_char_array_1[2] ? temp_char_array_1[2] : "(null)", 0));
		strcpy(temp_char_SVS_DATALEN_2, regsub("%3", temp_char_SVS_DATALEN_2,
			temp_char_array_1[3] ? temp_char_array_1[3] : "(null)", 0));
		strcpy(temp_char_SVS_DATALEN_2, regsub("%4", temp_char_SVS_DATALEN_2,
			temp_char_array_1[4] ? temp_char_array_1[4] : "(null)", 0));

		for (temp_char_pointer_1 = strtok(SVS_NOTIFY, " ");
		     temp_char_pointer_1;
		     temp_char_pointer_1 = strtok(NULL, " ")) {
			splitc(temp_char_SVS_DATALEN_1, temp_char_pointer_1, ',');
			while (temp_char_SVS_DATALEN_1[0]) {
				rmspace(temp_char_SVS_DATALEN_1);
				send_svs_message(temp_char_SVS_DATALEN_1, temp_char_SVS_DATALEN_2);
				splitc(temp_char_SVS_DATALEN_1, temp_char_pointer_1, ',');
			}
			rmspace(temp_char_pointer_1);
			send_svs_message(temp_char_pointer_1, temp_char_SVS_DATALEN_2);
		}
	}
	return;
}

static void send_svs_message(char *to, char *message) {
	FILE *file;
	char line[601], *line_pointer, *line_to, *line_from, *line_time;
	int found = 0;
	ContextNote(REPORT_BUG);
	if (!get_user_by_handle(userlist, to))
		return;
  file = fopen(svs_notefile, "r");
  if (file == NULL)
  	return;
  while (!feof(file)) {
    fgets(line, 600, file);
    if (line[strlen(line) - 1] == '\n')
      line[strlen(line) - 1] = 0;
    if (feof(file))
    	break;   /* Don't use the last line. */
    rmspace(line);
    if ((!line[0]) || (line[0] == '#') || (line[0] == ';'))
    	continue;
		line_pointer = line;
		line_to = newsplit(&line_pointer);
		if (egg_strcasecmp(line_to, to))
			continue;
    line_from = newsplit(&line_pointer);
    if (egg_strcasecmp(line_from, "SVS"))
    	continue;
	  line_time = newsplit(&line_pointer);
	  if (!strcasecmp(line_pointer, message)) {
	  	found = 1;
	  	break;
		}
	}
	if (found)
		return;
	add_note(to, "SVS", message, -1, 0);
	return;
}

static void start_svs() {
	add_hook(HOOK_DAILY, (Function) do_svs);
}

static void close_svs() {
	del_hook(HOOK_DAILY, (Function) do_svs);
	del_hook(HOOK_SECONDLY, (Function) do_svs_write);
	del_hook(HOOK_SECONDLY, (Function) do_svs_read);
}
