/*
 * Copyright (C) 2002  Carl M. Gregory - MC_8
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static int tcl_addgreet STDVAR {
  struct chanset_t *c;
  char code[2];
	ContextNote(REPORT_BUG);
	BADARGS(4, 4, " channel method message");
	if ((!(c = findchan_by_dname(argv[1]))
                                     && strncpy(code, "1", sizeof(code))) ||
  	  (findgreet(c->dname)           && strncpy(code, "2", sizeof(code))) ||
  	  (strcasecmp(argv[2], "PRIVMSG") && strcasecmp(argv[2], "NOTICE")
		                                 && strncpy(code, "3", sizeof(code)))) {
		switch (code[0]) {
		  case '1':
		  	Tcl_AppendResult(irp, "invalid channel: ", argv[1], NULL);
		  	break;
		  case '2':
		  	Tcl_AppendResult(irp, "0", NULL);
		  	return TCL_OK;
			case '3':
				Tcl_AppendResult(irp, "bad addgreet method \"", argv[2],
		                     "\": should be one of: PRIVMSG NOTICE", NULL);
		  	break;
	 	}
	 	return TCL_ERROR;
	}
	add_greet(argv[1], argv[3], !strcasecmp(argv[2], "NOTICE") ? 1 : 0);
	Tcl_AppendResult(irp, "1", NULL);
	return TCL_OK;
}

static int tcl_delgreet STDVAR {
	greet_t *greet;
	ContextNote(REPORT_BUG);
	BADARGS(2, 2, " channel");
  if (!findchan_by_dname(argv[1])) {
		Tcl_AppendResult(irp, "invalid channel: ", argv[1], NULL);
		return TCL_ERROR;
	}
	if (!(greet = findgreet(argv[1]))) {
		// No greet setup for specified channel.
		Tcl_AppendResult(irp, "0", NULL);
		return TCL_OK;
	}
	del_greet(greet);
	Tcl_AppendResult(irp, "1", NULL);
	return TCL_OK;
}

static int tcl_greets STDVAR {
	greet_t *tmp;
	char *out[3], *x;
	ContextNote(REPORT_BUG);
	BADARGS(1, 2, " ?channel?");
	if (argc == 2) {
	  if (!findchan_by_dname(argv[1])) {
			Tcl_AppendResult(irp, "invalid channel: ", argv[1], NULL);
			return TCL_ERROR;
		}
		if (!(tmp = findgreet(argv[1])))
			return TCL_OK;
		out[0] = tmp->channel;
		out[1] = tmp->method ? "NOTICE" : "PRIVMSG";
		out[2] = tmp->message;
		x = Tcl_Merge(3, out);
		Tcl_AppendElement(irp, x);
		Tcl_Free((char *) x);
	}	else {
		for (tmp = greetdata; tmp; tmp = tmp->next) {
			out[0] = tmp->channel;
			out[1] = tmp->method ? "NOTICE" : "PRIVMSG";
			out[2] = tmp->message;
			x = Tcl_Merge(3, out);
			Tcl_AppendElement(irp, x);
			Tcl_Free((char *) x);
		}
	}
	return TCL_OK;
}

static tcl_cmds my_tcl_cmds[] = {
	{"addgreet", tcl_addgreet},
	{"delgreet", tcl_delgreet},
	{"greets", tcl_greets},
	{0, 0}
};
